import os

import pytest


@pytest.fixture
def txn_hash():
    return "0x96823521c0c1999e4c1023279a1b4ef343649d37ac5f5b50a5fc7264064cb0ac"


@pytest.fixture
def contract_address():
    return "0xbBeb16aB5F13cd52bDa5d7AD49be4E7677231680"


@pytest.fixture
def owner_address():
    return "0x27566e752e56D403fB436C8b7031e7fcc75b6b4f"


@pytest.fixture
def owner_private_key():
    return os.environ["TEST_ACCOUNT_PRIVATE_KEY"]



# test use from python code

import pytest
from ape_ethereum.transactions import Receipt
from eth_account import Account

from ape_apeman import APE


@pytest.fixture
def check_url():
    def _check_url(url):
        assert isinstance(url, str)
        assert url.startswith("https://")

    return _check_url


@pytest.fixture
def check_receipt():
    def _check_receipt(receipt):
        assert isinstance(receipt, Receipt)

    return _check_receipt


@pytest.fixture
def check_block_number():
    def _check_block_number(block_number):
        assert isinstance(block_number, int)
        assert block_number > 1

    return _check_block_number


def test_module_context_url(txn_hash, check_url):
    with APE() as ape:
        url = ape.explorer.get_transaction_url(txn_hash)
    check_url(url)


def test_module_init_url(txn_hash, check_url):
    ape = APE(connect=True)
    url = ape.explorer.get_transaction_url(txn_hash)
    check_url(url)


def test_module_connect_url(txn_hash, check_url):
    ape = APE()
    ape.connect()
    url = ape.explorer.get_transaction_url(txn_hash)
    check_url(url)


def test_module_context_receipt(txn_hash, check_receipt):
    with APE() as ape:
        receipt = ape.provider.get_receipt(txn_hash)
    check_receipt(receipt)


def test_module_init_receipt(txn_hash, check_receipt):
    ape = APE(connect=True)
    receipt = ape.provider.get_receipt(txn_hash)
    check_receipt(receipt)


def test_module_connect_receipt(txn_hash, check_receipt):
    ape = APE()
    ape.connect()
    receipt = ape.provider.get_receipt(txn_hash)
    check_receipt(receipt)


def test_module_context_web3(check_block_number):
    with APE() as ape:
        n = ape.web3.eth.get_block_number()
    check_block_number(n)


def test_module_init_web3(check_block_number):
    ape = APE(connect=True)
    n = ape.web3.eth.get_block_number()
    check_block_number(n)


def test_module_connect_web3(check_block_number):
    ape = APE()
    ape.connect()
    n = ape.web3.eth.get_block_number()
    check_block_number(n)


def test_module_context_contract(contract_address):
    with APE() as ape:
        contract = ape.contracts.instance_at(contract_address)
        assert contract
        symbol = contract.symbol()
        assert symbol == "ETHERSIEVE"
        print(f"{contract_address=} {symbol=}")


def test_module_init_contract(contract_address):
    ape = APE(connect=True)
    contract = ape.contracts.instance_at(contract_address)
    assert contract
    symbol = contract.symbol()
    assert symbol == "ETHERSIEVE"
    print(f"{contract_address=} {symbol=}")


def test_module_connect_contract(contract_address):
    ape = APE()
    ape.connect()
    contract = ape.contracts.instance_at(contract_address)
    assert contract
    symbol = contract.symbol()
    assert symbol == "ETHERSIEVE"
    print(f"{contract_address=} {symbol=}")


def _sign_and_send(ape, contract_address, owner_address, owner_private_key):
    account = Account().from_key(owner_private_key)
    assert account.address == owner_address
    contract = ape.contracts.instance_at(contract_address)
    txn = contract.getPrices.as_transaction(
        sender=account.address, required_confirmations=1
    )
    txn.nonce = ape.provider.get_nonce(account.address)
    txn.signature = account.sign_transaction(txn.dict())
    receipt = ape.provider.send_transaction(txn)
    print(receipt)
    ret = receipt.return_value
    print(ret)


def test_module_context_sign_and_send(
    contract_address, owner_address, owner_private_key
):
    with APE() as ape:
        _sign_and_send(ape, contract_address, owner_address, owner_private_key)


def test_module_init_sign_and_send(
    contract_address, owner_address, owner_private_key
):
    ape = APE(connect=True)
    _sign_and_send(ape, contract_address, owner_address, owner_private_key)


def test_module_connect_sign_and_send(
    contract_address, owner_address, owner_private_key
):
    ape = APE()
    ape.connect()
    _sign_and_send(ape, contract_address, owner_address, owner_private_key)



#!/usr/bin/env python

import json
from traceback import print_exception

import pytest
from click.testing import CliRunner

import ape_apeman
from ape_apeman import __version__, cli


def test_cli_version():
    """Test reading version and module name"""
    assert ape_apeman.__name__ == "ape_apeman"
    assert __version__
    assert isinstance(__version__, str)


@pytest.fixture
def run():
    runner = CliRunner()

    # env = os.environ.copy()
    # env['EXTRA_ENV_VAR'] = 'VALUE'

    def _run(cmd, **kwargs):
        expect_exit_code = kwargs.pop("expect_exit_code", 0)
        expect_exception = kwargs.pop("expect_exception", None)
        if expect_exception is None:
            kwargs["catch_exceptions"] = False

        # kwargs["env"] = env
        result = runner.invoke(cli, cmd, **kwargs)
        if result.exception:
            if expect_exception and isinstance(
                result.exception, expect_exception
            ):
                return result
            else:
                print_exception(result.exception)
                breakpoint()
                pass
        else:
            assert result.exit_code == expect_exit_code, result.output
        if expect_exception:
            raise RuntimeError(f"unraised {expect_exception=}")
        return result

    return _run


def test_cli_no_args(run):
    """Test the CLI."""
    result = run([])
    assert "Usage:" in result.output


def test_cli_help(run):
    result = run(["--help"])
    assert "Show this message and exit." in result.output


def test_cli_txn_url(run, txn_hash):
    result = run(["txn", "--url", txn_hash])
    url = json.loads(result.output)
    assert isinstance(url, str)
    assert url.startswith("https://")


TXN_KEYS = [
    "block_number",
    "gas_used",
    "logs",
    "status",
    "txn_hash",
    "transaction",
    "gas_limit",
    "gas_price",
]


def test_cli_txn_json(run, txn_hash):
    result = run(["txn", txn_hash])
    txn = json.loads(result.output)
    assert isinstance(txn, dict)
    assert set(txn.keys()) == set(TXN_KEYS)



# ape context manager

import os

import ape


class APE:
    def __init__(
        self,
        ecosystem=None,
        network=None,
        provider=None,
        selector=None,
        connect=False,
    ):
        if selector is None:
            ecosystem = ecosystem or os.environ["APE_ECOSYSTEM"]
            network = network or os.environ["APE_NETWORK"]
            provider = provider or os.environ["APE_PROVIDER"]
            selector = os.environ.get(
                "APE_SELECTOR", f"{ecosystem}:{network}:{provider}"
            )
        self.context_manager = ape.networks.parse_network_choice(selector)
        self.connection = None
        if connect is True:
            self.connect()

    def __del__(self):
        if self.connection:
            self.disconnect()

    def connect(self, *args, **kwargs):
        if self.connection is None:
            self.connection = self.context_manager.__enter__(*args, **kwargs)
            self.provider = self.connection
            self.network = self.provider.network
            self.explorer = self.provider.network.explorer
            self.web3 = self.provider.web3
            self.contracts = self.network.chain_manager.contracts
        return self

    def disconnect(self, *args, **kwargs):
        if self.connection:
            self.context_manager.__exit__(*args, **kwargs)
        self.connection = None

    def __enter__(self, *args, **kwargs):
        return self.connect(*args, **kwargs)

    def __exit__(self, *args, **kwargs):
        self.disconnect(*args, **kwargs)



# hex or humanized json dumps

import json
from decimal import Decimal

from eth_utils import humanize_bytes, to_hex
from hexbytes import HexBytes


class JSONEncoder_hex_bytes(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Decimal):
            return str(o)
        elif isinstance(o, HexBytes):
            return to_hex(bytes(o))
        elif isinstance(o, bytes):
            return to_hex(o)
        else:
            return super().default(o)


class JSONEncoder_humanize(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Decimal):
            return str(o)
        elif isinstance(o, HexBytes):
            return humanize_bytes(bytes(o))
        elif isinstance(o, bytes):
            return humanize_bytes(o)
        else:
            return super().default(o)


def dumps(*args, **kwargs):
    humanize = kwargs.pop("humanize", False)
    hex_bytes = kwargs.pop("hex_bytes", False)
    if humanize:
        kwargs["cls"] = JSONEncoder_humanize
    elif hex_bytes:
        kwargs["cls"] = JSONEncoder_hex_bytes

    return json.dumps(*args, **kwargs)
